<!DOCTYPE html>
{% macro action_button(addr, name, display) %}
    <button type="submit" name='action' value="{{ name }}"
            formaction="{{ url_for('action', host=addr[0], port=addr[1]) }}">
        {{ display }}</button>
{% endmacro -%}
{% macro action_button_all(type, name, display) %}
    <button type="submit" name='action' value="{{ name }}"
            formaction="{{ url_for('action_all', type=type) }}">
        {{ display }}</button>
{% endmacro -%}
<html>
<head>
    <title>Kuyruk Manager</title>
    <link rel="shortcut icon" type="image/x-icon"
          href="{{ url_for('static', filename='favicon.ico') }}"/>
    <style type="text/css">
        table {border-collapse: collapse; }
        table, th, td {border: 1px solid black; }
    </style>
</head>
<body>
    <nav>
        <a href="{{ url_for('masters') }}">Masters</a> |
        <a href="{{ url_for('workers') }}">Workers</a> |
        <a href="{{ url_for('queues') }}">Queues</a> |
        <a href="{{ url_for('failed_tasks') }}">Failed Tasks</a>
    </nav>
    <br>
    {% block body %}
    {% endblock %}
    <br>
    <footer>
        <hr>
        {{ now }} - Kuyruk Manager {{ version }} is running on host: {{ hostname }}
    </footer>
</body>
</html>
