{% extends "layout.html" %}

{% block body %}
    <form method="POST">
        <table>
            <tr>
                <th>Queue</th>
                <th>Messages Ready</th>
                <th>Consumers</th>
            </tr>
            {% for queue in queues|sort(attribute='name') %}
            <tr>
                <td>{{ queue.name }}</td>
                <td>{{ queue.messages_ready }}</td>
                <td>{{ queue.consumers }}</td>
            </tr>
            {% else %}
            {% endfor %}
        </table>
        Count: {{ queues|count }}
    </form>
{% endblock %}
